import { writable, derived, get } from 'svelte/store';
import { ZodError } from 'zod';

function zip(...arrays) {
  if (!arrays || !arrays.length)
    return [];
  return new Array(Math.max(...arrays.map(({ length }) => length))).fill([]).map((_, idx) => arrays.map((array) => array[idx]));
}

const debounce = ({ delay }, func) => {
  let timer = void 0;
  let active = true;
  const debounced = (...args) => {
    if (active) {
      clearTimeout(timer);
      timer = setTimeout(() => {
        active && func(...args);
      }, delay);
    } else {
      func(...args);
    }
  };
  debounced.cancel = () => {
    active = false;
  };
  debounced.flush = (...args) => func(...args);
  return debounced;
};

const trurthly = (z) => !!z;
const toReadable = (w) => derived(w, a => a);
const getErrorMessage = (e) => {
    if (e instanceof ZodError)
        return e.format()._errors.join(',');
    if (e instanceof Error)
        return e.message;
    else if (typeof e === 'string')
        return e;
    else
        return JSON.stringify(e);
};
class ZodFormStore {
    model;
    fields;
    triggerSubmit;
    reset;
    submitting;
    errors;
    valid;
    constructor(schema, options) {
        const { initialValue, onSubmit, debounceDelay } = options || {};
        // Form stores
        const submitting = writable(false);
        const formError = writable();
        // Fields stores
        const fieldNames = Object.keys(schema.shape);
        const fieldStores = fieldNames.map(name => new ZodFieldStore(schema, name, initialValue?.[name], debounceDelay));
        const model = derived(fieldStores.map(z => z.value), z => Object.fromEntries(zip(fieldNames, z)));
        const fieldErrors = derived(fieldStores.map(z => z.error), e => e.filter(trurthly));
        const errors = derived([fieldErrors, formError], errors => errors.flatMap(z => z).filter(trurthly));
        const valid = derived(errors, e => !e);
        const triggerSubmit = async () => {
            if (!onSubmit)
                return;
            const _fieldErrors = get(fieldErrors);
            if (_fieldErrors.length > 0) {
                console.log('❗️ Please resolve field errors before submitting again', _fieldErrors);
                return;
            }
            formError.set('');
            submitting.set(true);
            try {
                const values = schema.parse(get(model));
                const result = await onSubmit(values);
                if (result)
                    formError.set(result);
            }
            catch (e) {
                console.log(`🚫 Form error`, e);
                formError.set(getErrorMessage(e));
            }
            finally {
                submitting.set(false);
            }
        };
        function handleReset() {
            submitting.set(false);
            formError.set('');
            fieldStores.forEach(fs => fs.reset());
        }
        this.fields = Object.fromEntries(fieldStores.map(z => [z.name, z]));
        this.triggerSubmit = triggerSubmit;
        this.reset = handleReset;
        this.submitting = toReadable(submitting);
        this.errors = errors;
        this.model = model;
        this.valid = valid;
    }
}
class ZodFieldStore {
    name;
    value;
    touched;
    dirty;
    error;
    handleUpdate;
    handleChange;
    handleBlur;
    reset;
    constructor(parentSchema, name, initialValue, ms = 0) {
        // Stores
        const touched = writable(false);
        const dirty = writable(false);
        const error = writable('');
        const value = writable();
        const setError = ms > 0 ? debounce({ delay: ms }, error.set) : error.set;
        const setTouched = ms > 0 ? debounce({ delay: ms }, touched.set) : touched.set;
        const setDirty = ms > 0 ? debounce({ delay: ms }, dirty.set) : dirty.set;
        const schema = parentSchema.shape[name];
        const handleError = (e) => setError(getErrorMessage(e));
        try {
            initialValue = schema.parse(initialValue);
        }
        catch (e) {
            handleError(e);
        }
        const resetValue = () => value.set(initialValue);
        resetValue();
        const setValue = ms > 0 ? debounce({ delay: ms }, value.set) : value.set;
        const updateValue = ms > 0 ? debounce({ delay: ms }, value.update) : value.update;
        const handleChange = (e) => {
            setError('');
            let nextVal = e;
            if (e instanceof Event)
                if (e instanceof CustomEvent)
                    if (typeof e.detail === 'object' && 'value' in e.detail)
                        nextVal = e.detail.value;
                    else
                        nextVal = e.detail;
                else if (e.target instanceof HTMLInputElement)
                    if (e.target.type === 'checkbox')
                        nextVal = e.target.checked;
                    else
                        nextVal = e.target.value;
                else if (e.target instanceof HTMLSelectElement)
                    nextVal = e.target.value;
                else if (e.target instanceof HTMLTextAreaElement)
                    nextVal = e.target.value;
            try {
                nextVal = schema.parse(nextVal);
            }
            catch (e) {
                handleError(e);
            }
            setValue(nextVal);
            setTouched(true);
            setDirty(true);
        };
        const handleUpdate = (updater) => {
            const safeUpdater = v => {
                setError('');
                const updatedVal = updater(v);
                if (schema)
                    try {
                        return schema.parse(updatedVal);
                    }
                    catch (e) {
                        handleError(e);
                    }
                return updatedVal;
            };
            updateValue(safeUpdater);
            setTouched(true);
            setDirty(true);
        };
        const handleBlur = () => setTouched(true);
        const handleReset = () => {
            touched.set(false);
            dirty.set(false);
            error.set('');
            resetValue();
        };
        this.name = name;
        this.value = toReadable(value);
        this.touched = toReadable(touched);
        this.dirty = toReadable(dirty);
        this.error = derived([error, touched], ([e, t]) => (t ? e : ''));
        this.handleChange = handleChange;
        this.handleUpdate = handleUpdate;
        this.handleBlur = handleBlur;
        this.reset = handleReset;
    }
}

export { ZodFieldStore, ZodFormStore };
//# sourceMappingURL=index.mjs.map
